#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought naming package component>
#------------------------------------------------------------------------------
""" Tests naming operations on PyFS contexts. """


# Standard library imports.
import os, shutil, unittest

# Enthought library imports.
from apptools.io import File
from apptools.naming.api import *


class PyFSContextTestCase(unittest.TestCase):
    """ Tests naming operations on PyFS contexts. """

    ###########################################################################
    # 'TestCase' interface.
    ###########################################################################

    def setUp(self):
        """ Prepares the test fixture before each test method is called. """

        try:
            if os.path.exists('data'): shutil.rmtree('data')
            if os.path.exists('other'): shutil.rmtree('other')

        except:
            pass

        os.mkdir('data')
        os.mkdir('other')

        self.context = PyFSContext(path='data')
        self.context.create_subcontext('sub')
        self.context.bind('x', 123)
        self.context.bind('y', 321)

        return

    def tearDown(self):
        """ Called immediately after each test method has been called. """

        self.context = None

        shutil.rmtree('data')
        shutil.rmtree('other')

        return

    ###########################################################################
    # Tests.
    ###########################################################################

    def test_initialization(self):
        """ initialization of an existing context """

        context = PyFSContext(path='data')
        self.assertEqual(len(context.list_bindings('')), 3)

        return

    def test_initialization_with_empty_environment(self):
        """ initialization with empty environmentt """

        context = PyFSContext(path='other', environment={})
        self.assertEqual(len(context.list_names('')), 0)

        return

    def test_bind(self):
        """ pyfs context bind """

        # Convenience.
        context = self.context
        sub = self.context.lookup('sub')

        # Make sure that the sub-context is empty.
        self.assertEqual(len(sub.list_bindings('')), 0)

        # Empty name.
        self.assertRaises(InvalidNameError, context.bind, '', 1)

        # Bind a local file object.
        f = File(os.path.join(sub.path, 'foo.py'))
        #f.create_file('print "foo!"\n')

        context.bind('sub/foo.py', f)
        self.assertEqual(len(sub.list_bindings('')), 1)

        # Bind a reference to a non-local file.
        f = File('/tmp')
        context.bind('sub/tmp', f)
        self.assertEqual(len(sub.list_bindings('')), 2)
        self.assertEqual(context.lookup('sub/tmp').path, f.path)

        # Bind a reference to a non-local context.
        f = PyFSContext(path='other')
        context.bind('sub/other', f)
        self.assertEqual(len(sub.list_bindings('')), 3)
        self.assert_(f.path in context.lookup('sub/other').path)

        # Bind a Python object.
        context.bind('sub/a', 1)
        self.assertEqual(len(sub.list_bindings('')), 4)

        # Try to bind it again.
        self.assertRaises(NameAlreadyBoundError, context.bind, 'sub/a', 1)

        return

    def test_rebind(self):
        """ pyfs context rebind """

        # Convenience.
        context = self.context
        sub = self.context.lookup('sub')

        # Make sure that the sub-context is empty.
        self.assertEqual(len(sub.list_bindings('')), 0)

        # Empty name.
        self.assertRaises(InvalidNameError, context.rebind, '', 1)

        # Bind a name.
        context.bind('sub/a', 1)
        self.assertEqual(len(sub.list_bindings('')), 1)

        # Rebind it.
        context.rebind('sub/a', 1)
        self.assertEqual(len(sub.list_bindings('')), 1)

        return

    def test_unbind(self):
        """ pyfs context unbind """

        # Convenience.
        context = self.context
        sub = self.context.lookup('sub')

        # Make sure that the sub-context is empty.
        self.assertEqual(len(sub.list_bindings('')), 0)

        # Empty name.
        self.assertRaises(InvalidNameError, context.unbind, '')

        # Bind a name.
        context.bind('sub/a', 1)
        self.assertEqual(len(sub.list_bindings('')), 1)

        # Unbind it.
        context.unbind('sub/a')
        self.assertEqual(len(sub.list_bindings('')), 0)

        # Try to unbind a non-existent name.
        self.assertRaises(NameNotFoundError, context.unbind, 'sub/b')

        return

    def test_rename(self):
        """ multi-context rename """

        # Convenience.
        context = self.context
        sub = self.context.lookup('sub')

        # Make sure that the sub-context is empty.
        self.assertEqual(len(sub.list_bindings('')), 0)

        # Empty name.
        self.assertRaises(InvalidNameError, context.rename, '', 'x')
        self.assertRaises(InvalidNameError, context.rename, 'x', '')

        # Bind a name.
        context.bind('sub/a', 1)
        self.assertEqual(len(sub.list_bindings('')), 1)

        # Rename it.
        context.rename('sub/a', 'sub/b')
        self.assertEqual(len(sub.list_bindings('')), 1)

        # Lookup using the new name.
        self.assertEqual(context.lookup('sub/b'), 1)

        # Lookup using the old name.
        self.assertRaises(NameNotFoundError, context.lookup, 'sub/a')

    def test_lookup(self):
        """ pyfs context lookup """

        # Convenience.
        context = self.context
        sub = self.context.lookup('sub')

        # Make sure that the sub-context is empty.
        self.assertEqual(len(sub.list_bindings('')), 0)

        # Bind a file object.
        f = File(os.path.join(sub.path, 'foo.py'))
        #f.create_file('print "foo!"\n')

        context.bind('sub/foo.py', f)
        self.assertEqual(len(sub.list_bindings('')), 1)

        # Look it up.
        self.assertEqual(context.lookup('sub/foo.py').path, f.path)

        # Bind a Python object.
        context.bind('sub/a', 1)
        self.assertEqual(len(sub.list_bindings('')), 2)

        # Look it up.
        self.assertEqual(context.lookup('sub/a'), 1)

        # Looking up the Empty name returns the context itself.
        self.assertEqual(context.lookup(''), context)

        # Non-existent name.
        self.assertRaises(NameNotFoundError, context.lookup, 'sub/b')

        return

    def test_create_subcontext(self):
        """ pyfs context create sub-context """

        # Convenience.
        context = self.context
        sub = self.context.lookup('sub')

        # Make sure that the sub-context is empty.
        self.assertEqual(len(sub.list_bindings('')), 0)

        # Empty name.
        self.assertRaises(InvalidNameError, context.create_subcontext, '')

        # Create a sub-context.
        a = context.create_subcontext('sub/a')
        self.assertEqual(len(sub.list_bindings('')), 1)
        self.assert_(os.path.isdir(os.path.join(sub.path, 'a')))

        # Try to bind it again.
        self.assertRaises(
            NameAlreadyBoundError, context.create_subcontext, 'sub/a'
        )

        return

    def test_destroy_subcontext(self):
        """ single context destroy sub-context """

        # Convenience.
        context = self.context
        sub = self.context.lookup('sub')

        # Make sure that the sub-context is empty.
        self.assertEqual(len(sub.list_bindings('')), 0)

        # Empty name.
        self.assertRaises(InvalidNameError, context.destroy_subcontext, '')

        # Create a sub-context.
        a = context.create_subcontext('sub/a')
        self.assertEqual(len(sub.list_bindings('')), 1)

        # Destroy it.
        context.destroy_subcontext('sub/a')
        self.assertEqual(len(sub.list_bindings('')), 0)
        self.assert_(not os.path.isdir(os.path.join(sub.path, 'a')))

        # Bind a name.
        context.bind('sub/a', 1)
        self.assertEqual(len(sub.list_bindings('')), 1)

        # Try to destroy it.
        self.assertRaises(
            NotContextError, context.destroy_subcontext, 'sub/a'
        )

        return

        # Try to destroy a non-existent name.
        self.assertRaises(
            NameNotFoundError, context.destroy_subcontext, 'sub/b'
        )

        return

    def test_get_attributes(self):
        """ get attributes """

        # Convenience.
        context = self.context
        sub = self.context.lookup('sub')
        self.assert_(isinstance(sub, DirContext))

        #### Generic name resolution tests ####

        # Non-existent name.
        self.assertRaises(NameNotFoundError, context.get_attributes, 'xx')

        # Attempt to resolve via a non-existent context.
        self.assertRaises(NameNotFoundError, context.get_attributes,'xx/a')

        # Attempt to resolve via an existing name that is not a context.
        context.bind('sub/a', 1)
        self.assertEqual(len(sub.list_bindings('')), 1)
        self.assertRaises(NotContextError,context.get_attributes,'sub/a/x')

        #### Operation specific tests ####

        # Attributes of the root context.
        attributes = context.get_attributes('')
        self.assertEqual(len(attributes), 0)

        # Attributes of a sub-context.
        attributes = context.get_attributes('sub')
        self.assertEqual(len(attributes), 0)

        return

    def test_set_get_attributes(self):
        """ get and set attributes """

        defaults = {'colour' : 'blue'}

        # Convenience.
        context = self.context
        sub = self.context.lookup('sub')
        self.assert_(isinstance(sub, DirContext))

        #### Generic name resolution tests ####

        # Non-existent name.
        self.assertRaises(
            NameNotFoundError, context.set_attributes, 'xx', defaults
        )

        # Attempt to resolve via a non-existent context.
        self.assertRaises(
            NameNotFoundError, context.set_attributes, 'xx/a', defaults
        )

        # Attempt to resolve via an existing name that is not a context.
        context.bind('sub/a', 1)
        self.assertEqual(len(sub.list_bindings('')), 1)
        self.assertRaises(
            NotContextError, context.set_attributes, 'sub/a/xx', defaults
        )

        #### Operation specific tests ####

        # Attributes of the root context.
        attributes = self.context.get_attributes('')
        self.assertEqual(len(attributes), 0)

        # Set the attributes.
        context.set_attributes('', defaults)
        attributes = context.get_attributes('')
        self.assertEqual(len(attributes), 1)
        self.assertEqual(attributes['colour'], 'blue')

        # Attributes of a sub-context.
        attributes = context.get_attributes('sub')
        self.assertEqual(len(attributes), 0)

        # Set the attributes.
        context.set_attributes('sub', defaults)
        attributes = context.get_attributes('sub')
        self.assertEqual(len(attributes), 1)
        self.assertEqual(attributes['colour'], 'blue')

        return

    def test_namespace_name(self):
        """ get the name of a context within its namespace. """

        # Convenience.
        context = self.context
        sub = self.context.lookup('sub')
        self.assert_(isinstance(sub, DirContext))

        self.assertEqual(context.namespace_name, 'data')
        self.assertEqual(sub.namespace_name, 'data/sub')

        return

#### EOF ######################################################################



""" Attach relevant project files.

FIXME: there are no public project plugins for Envisage 3, yet. In any case,
this stuff should not be hard-coded, but extensible via extension points. The
code remains here because we can reuse the zip utility code in that extensible
rewrite.
"""

import logging
import os.path
from email import Encoders
from email.MIMEBase import MIMEBase

from traits.api import Any, HasTraits


logger = logging.getLogger(__name__)


class Attachments(HasTraits):

    application = Any()
    message = Any()

    def __init__(self, message, **traits):
        traits = traits.copy()
        traits['message'] = message
        super(Attachments, self).__init__(**traits)


    # FIXME: all of the package_*() methods refer to deprecated project plugins.

    def package_workspace(self):
        if self.application is None:
            pass

        workspace = self.application.get_service('envisage.project.IWorkspace')
        if workspace is not None:
            dir = workspace.path
            self._attach_directory(dir)
        return

    def package_single_project(self):
        if self.application is None:
            pass

        single_project = self.application.get_service('envisage.single_project.ModelService')
        if single_project is not None:
            dir = single_project.location
            self._attach_directory(dir)

    def package_any_relevant_files(self):
        self.package_workspace()
        self.package_single_project()
        return

    def _attach_directory(self, dir):
        relpath = os.path.basename(dir)

        import zipfile
        from io import StringIO

        ctype = 'application/octet-stream'
        maintype, subtype = ctype.split('/', 1)
        msg = MIMEBase(maintype, subtype)

        file_object = StringIO()
        zip = zipfile.ZipFile(file_object, 'w')
        _append_to_zip_archive(zip, dir, relpath)
        zip.close()

        msg.set_payload(file_object.getvalue())

        Encoders.encode_base64(msg) # Encode the payload using Base64
        msg.add_header('Content-Disposition', 'attachment', filename='project.zip')

        self.message.attach(msg)

        file_object.close()




def _append_to_zip_archive(zip, dir, relpath):
    """ Add all files in and below directory dir into zip archive"""
    for filename in os.listdir(dir):
        path = os.path.join(dir, filename)

        if os.path.isfile(path):
            name = os.path.join(relpath, filename)
            zip.write(path, name)
            logger.debug('adding %s to error report' % path)
        else:
            if filename != ".svn": # skip svn files if any
                subdir = os.path.join(dir, filename)
                _append_to_zip_archive(zip, subdir, os.path.join(relpath, filename))
    return




#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought IO package component>
#------------------------------------------------------------------------------
""" Tests folder operations. """


# Standard library imports.
import os, shutil, stat, unittest
from os.path import join

# Enthought library imports.
from apptools.io import File


class FolderTestCase(unittest.TestCase):
    """ Tests folder operations on a local file system. """

    ###########################################################################
    # 'TestCase' interface.
    ###########################################################################

    def setUp(self):
        """ Prepares the test fixture before each test method is called. """

        try:
            shutil.rmtree('data')

        except:
            pass

        os.mkdir('data')

        return

    def tearDown(self):
        """ Called immediately after each test method has been called. """

        shutil.rmtree('data')

        return

    ###########################################################################
    # Tests.
    ###########################################################################

    def test_properties(self):
        """ folder properties """

        # Properties of a non-existent folder.
        f = File('data/bogus')

        self.assert_(os.path.abspath(os.path.curdir) in f.absolute_path)
        self.assertEqual(f.children, None)
        self.assertEqual(f.ext, '')
        self.assertEqual(f.exists, False)
        self.assertEqual(f.is_file, False)
        self.assertEqual(f.is_folder, False)
        self.assertEqual(f.is_package, False)
        self.assertEqual(f.is_readonly, False)
        self.assertEqual(f.mime_type, 'content/unknown')
        self.assertEqual(f.name, 'bogus')
        self.assertEqual(f.parent.path, 'data')
        self.assertEqual(f.path, 'data/bogus')
        self.assert_(os.path.abspath(os.path.curdir) in f.url)
        self.assertEqual(str(f), 'File(%s)' % f.path)

        # Properties of an existing folder.
        f = File('data/sub')
        f.create_folder()

        self.assert_(os.path.abspath(os.path.curdir) in f.absolute_path)
        self.assertEqual(len(f.children), 0)
        self.assertEqual(f.ext, '')
        self.assertEqual(f.exists, True)
        self.assertEqual(f.is_file, False)
        self.assertEqual(f.is_folder, True)
        self.assertEqual(f.is_package, False)
        self.assertEqual(f.is_readonly, False)
        self.assertEqual(f.mime_type, 'content/unknown')
        self.assertEqual(f.name, 'sub')
        self.assertEqual(f.parent.path, 'data')
        self.assertEqual(f.path, 'data/sub')
        self.assert_(os.path.abspath(os.path.curdir) in f.url)

        # Make it readonly.
        os.chmod(f.path, stat.S_IRUSR)
        self.assertEqual(f.is_readonly, True)

        # And then make it NOT readonly so that we can delete it at the end of
        # the test!
        os.chmod(f.path, stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR)
        self.assertEqual(f.is_readonly, False)

        # Properties of a Python package folder.
        f = File('data/package')
        f.create_folder()

        init = File('data/package/__init__.py')
        init.create_file()

        self.assert_(os.path.abspath(os.path.curdir) in f.absolute_path)
        self.assertEqual(len(f.children), 1)
        self.assertEqual(f.ext, '')
        self.assertEqual(f.exists, True)
        self.assertEqual(f.is_file, False)
        self.assertEqual(f.is_folder, True)
        self.assertEqual(f.is_package, True)
        self.assertEqual(f.is_readonly, False)
        self.assertEqual(f.mime_type, 'content/unknown')
        self.assertEqual(f.name, 'package')
        self.assertEqual(f.parent.path, 'data')
        self.assertEqual(f.path, 'data/package')
        self.assert_(os.path.abspath(os.path.curdir) in f.url)

        return

    def test_copy(self):
        """ folder copy """

        f = File('data/sub')
        self.assertEqual(f.exists, False)

        # Create the folder.
        f.create_folder()
        self.assertEqual(f.exists, True)
        self.assertRaises(ValueError, f.create_folder)

        self.assertEqual(len(f.children), 0)
        self.assertEqual(f.ext, '')
        self.assertEqual(f.is_file, False)
        self.assertEqual(f.is_folder, True)
        self.assertEqual(f.mime_type, 'content/unknown')
        self.assertEqual(f.name, 'sub')
        self.assertEqual(f.path, 'data/sub')

        # Copy the folder.
        g = File('data/copy')
        self.assertEqual(g.exists, False)

        f.copy(g)
        self.assertEqual(g.exists, True)

        self.assertEqual(len(g.children), 0)
        self.assertEqual(g.ext, '')
        self.assertEqual(g.is_file, False)
        self.assertEqual(g.is_folder, True)
        self.assertEqual(g.mime_type, 'content/unknown')
        self.assertEqual(g.name, 'copy')
        self.assertEqual(g.path, 'data/copy')

        # Attempt to copy a non-existent folder (should do nothing).
        f = File('data/bogus')
        self.assertEqual(f.exists, False)

        g = File('data/bogus_copy')
        self.assertEqual(g.exists, False)

        f.copy(g)
        self.assertEqual(g.exists, False)

        return

    def test_create_folder(self):
        """ folder creation """

        f = File('data/sub')
        self.assertEqual(f.exists, False)

        # Create the folder.
        f.create_folder()
        self.assertEqual(f.exists, True)

        parent = File('data')
        self.assertEqual(len(parent.children), 1)
        self.assertEqual(parent.children[0].path, join('data', 'sub'))

        # Try to create it again.
        self.assertRaises(ValueError, f.create_folder)

        return

    def test_create_folders(self):
        """ nested folder creation """

        f = File('data/sub/foo')
        self.assertEqual(f.exists, False)

        # Attempt to create the folder with 'create_folder' which requires
        # that all intermediate folders exist.
        self.assertRaises(OSError, f.create_folder)

        # Create the folder.
        f.create_folders()
        self.assertEqual(f.exists, True)
        self.assertEqual(File('data/sub').exists, True)

        # Try to create it again.
        self.assertRaises(ValueError, f.create_folders)

        return

    def test_delete(self):
        """ folder deletion """

        f = File('data/sub')
        self.assertEqual(f.exists, False)

        # Create the folder.
        f.create_folder()
        self.assertEqual(f.exists, True)
        self.assertRaises(ValueError, f.create_folder)

        self.assertEqual(len(f.children), 0)
        self.assertEqual(f.ext, '')
        self.assertEqual(f.is_file, False)
        self.assertEqual(f.is_folder, True)
        self.assertEqual(f.mime_type, 'content/unknown')
        self.assertEqual(f.name, 'sub')
        self.assertEqual(f.path, 'data/sub')

        # Delete it.
        f.delete()
        self.assertEqual(f.exists, False)

        # Attempt to delete a non-existet folder (should do nothing).
        f = File('data/bogus')
        self.assertEqual(f.exists, False)

        f.delete()
        self.assertEqual(f.exists, False)

        return

#### EOF ######################################################################



"""Unit tests for the state pickler and unpickler.

"""
# Author: Prabhu Ramachandran <prabhu_r@users.sf.net>
# Copyright (c) 2005-2015, Enthought, Inc.
# License: BSD Style.

import base64
import unittest
import math

import numpy

from traits.api import Bool, Int, Long, Array, Float, Complex, Any, \
    Str, Unicode, Instance, Tuple, List, Dict, HasTraits

try:
    from tvtk.api import tvtk
except ImportError:
    TVTK_AVAILABLE = False
else:
    TVTK_AVAILABLE = True

from apptools.persistence import state_pickler


# A simple class to test instances.
class A(object):

    def __init__(self):
        self.a = 'a'

# NOTE: I think that TVTK specific testing should be moved to the
#       TVTK package.


# A classic class for testing the pickler.
class TestClassic:

    def __init__(self):
        self.b = False
        self.i = 7
        self.l = 1234567890123456789
        self.f = math.pi
        self.c = complex(1.01234, 2.3)
        self.n = None
        self.s = 'String'
        self.u = u'Unicode'
        self.inst = A()
        self.tuple = (1, 2, 'a', A())
        self.list = [1, 1.1, 'a', 1j, self.inst]
        self.pure_list = list(range(5))
        self.dict = {'a': 1, 'b': 2, 'ref': self.inst}
        self.numeric = numpy.ones((2, 2, 2), 'f')
        self.ref = self.numeric
        if TVTK_AVAILABLE:
            self._tvtk = tvtk.Property()


# A class with traits for testing the pickler.
class TestTraits(HasTraits):
    b = Bool(False)
    i = Int(7)
    l = Long(12345678901234567890)
    f = Float(math.pi)
    c = Complex(complex(1.01234, 2.3))
    n = Any
    s = Str('String')
    u = Unicode(u'Unicode')
    inst = Instance(A)
    tuple = Tuple
    list = List
    pure_list = List(list(range(5)))
    dict = Dict
    numeric = Array(value=numpy.ones((2, 2, 2), 'f'))
    ref = Array
    if TVTK_AVAILABLE:
        _tvtk = Instance(tvtk.Property, ())

    def __init__(self):
        self.inst = A()
        self.tuple = (1, 2, 'a', A())
        self.list = [1, 1.1, 'a', 1j, self.inst]
        self.dict = {'a': 1, 'b': 2, 'ref': self.inst}
        self.ref = self.numeric


class TestDictPickler(unittest.TestCase):

    def set_object(self, obj):
        """Changes the objects properties to test things."""
        obj.b = True
        obj.i = 8
        obj.s = 'string'
        obj.u = u'unicode'
        obj.inst.a = 'b'
        obj.list[0] = 2
        obj.tuple[-1].a = 't'
        obj.dict['a'] = 10
        if TVTK_AVAILABLE:
            obj._tvtk.trait_set(
                point_size=3, specular_color=(1, 0, 0),
                representation='w'
            )

    def _check_instance_and_references(self, obj, data):
        """Asserts that there is one instance and two references in the state.
        We need this as there isn't a guarantee as to which will be the
        reference and which will be the instance.
        """
        inst = data['inst']
        list_end = data['list']['data'][-1]
        dict_ref = data['dict']['data']['ref']
        all_inst = [inst, list_end, dict_ref]
        types = [x['type'] for x in all_inst]
        self.assertEqual(types.count('instance'), 1)
        self.assertEqual(types.count('reference'), 2)
        inst_state = all_inst[types.index('instance')]
        self.assertEqual(inst_state['data']['data']['a'], 'b')

    def verify(self, obj, state):
        data = state['data']
        self.assertEqual(state['class_name'], obj.__class__.__name__)
        data = data['data']
        self.assertEqual(data['b'], obj.b)
        self.assertEqual(data['i'], obj.i)
        self.assertEqual(data['l'], obj.l)
        self.assertEqual(data['f'], obj.f)
        self.assertEqual(data['c'], obj.c)
        self.assertEqual(data['n'], obj.n)
        self.assertEqual(data['s'], obj.s)
        self.assertEqual(data['u'], obj.u)
        tup = data['tuple']['data']
        self.assertEqual(tup[:-1], obj.tuple[:-1])
        self.assertEqual(tup[-1]['data']['data']['a'], 't')
        lst = data['list']['data']
        self.assertEqual(lst[:-1], obj.list[:-1])

        pure_lst = data['pure_list']['data']
        self.assertEqual(pure_lst, obj.pure_list)
        dct = data['dict']['data']
        self.assertEqual(dct['a'], obj.dict['a'])
        self.assertEqual(dct['b'], obj.dict['b'])

        self._check_instance_and_references(obj, data)

        num_attr = 'numeric' if data['numeric']['type'] == 'numeric' else 'ref'
        decodestring = getattr(base64, 'decodebytes', base64.decodestring)
        junk = state_pickler.gunzip_string(
            decodestring(data[num_attr]['data'])
        )
        num = numpy.loads(junk)
        self.assertEqual(numpy.alltrue(numpy.ravel(num == obj.numeric)), 1)

        self.assertTrue(data['ref']['type'] in ['reference', 'numeric'])
        if data['ref']['type'] == 'numeric':
            self.assertEqual(data['numeric']['type'], 'reference')
        else:
            self.assertEqual(data['numeric']['type'], 'numeric')
        self.assertEqual(data['ref']['id'], data['numeric']['id'])

    def verify_unpickled(self, obj, state):
        self.assertEqual(state.__metadata__['class_name'],
                         obj.__class__.__name__)
        self.assertEqual(state.b, obj.b)
        self.assertEqual(state.i, obj.i)
        self.assertEqual(state.l, obj.l)
        self.assertEqual(state.f, obj.f)
        self.assertEqual(state.c, obj.c)
        self.assertEqual(state.n, obj.n)
        self.assertEqual(state.s, obj.s)
        self.assertEqual(state.u, obj.u)
        self.assertEqual(state.inst.__metadata__['type'], 'instance')

        tup = state.tuple
        self.assertEqual(state.tuple.has_instance, True)
        self.assertEqual(tup[:-1], obj.tuple[:-1])
        self.assertEqual(tup[-1].a, 't')
        lst = state.list
        self.assertEqual(state.list.has_instance, True)
        self.assertEqual(lst[:-1], obj.list[:-1])
        # Make sure the reference is the same
        self.assertEqual(id(state.inst), id(lst[-1]))

        self.assertEqual(lst[-1].a, 'b')

        pure_lst = state.pure_list
        self.assertEqual(pure_lst, obj.pure_list)
        self.assertEqual(state.pure_list.has_instance, False)

        dct = state.dict
        self.assertEqual(dct.has_instance, True)
        self.assertEqual(dct['a'], obj.dict['a'])
        self.assertEqual(dct['b'], obj.dict['b'])
        self.assertEqual(dct['ref'].__metadata__['type'], 'instance')

        num = state.numeric
        self.assertEqual(numpy.alltrue(numpy.ravel(num == obj.numeric)), 1)
        self.assertEqual(id(state.ref), id(num))

        if TVTK_AVAILABLE:
            _tvtk = state._tvtk
            self.assertEqual(_tvtk.representation, obj._tvtk.representation)
            self.assertEqual(_tvtk.specular_color, obj._tvtk.specular_color)
            self.assertEqual(_tvtk.point_size, obj._tvtk.point_size)

    def verify_state(self, state1, state):
        self.assertEqual(state.__metadata__,
                         state1.__metadata__)
        self.assertEqual(state.b, state1.b)
        self.assertEqual(state.i, state1.i)
        self.assertEqual(state.l, state1.l)
        self.assertEqual(state.f, state1.f)
        self.assertEqual(state.c, state1.c)
        self.assertEqual(state.n, state1.n)
        self.assertEqual(state.s, state1.s)
        self.assertEqual(state.u, state1.u)
        # The ID's need not be identical so we equate them here so the
        # tests pass.  Note that the ID's only need be consistent not
        # identical!
        if TVTK_AVAILABLE:
            instances = ('inst', '_tvtk')
        else:
            instances = ('inst',)

        for attr in instances:
            getattr(state1, attr).__metadata__['id'] = \
                getattr(state, attr).__metadata__['id']

        if TVTK_AVAILABLE:
            self.assertEqual(state1._tvtk, state._tvtk)

        state1.tuple[-1].__metadata__['id'] = \
            state.tuple[-1].__metadata__['id']
        self.assertEqual(state.inst.__metadata__, state1.inst.__metadata__)

        self.assertEqual(state.tuple, state1.tuple)
        self.assertEqual(state.list, state1.list)

        self.assertEqual(state.pure_list, state1.pure_list)

        self.assertEqual(state.dict, state1.dict)

        self.assertEqual((state1.numeric == state.numeric).all(), True)
        self.assertEqual(id(state.ref), id(state.numeric))
        self.assertEqual(id(state1.ref), id(state1.numeric))


    def test_has_instance(self):
        """Test to check has_instance correctness."""
        a = A()
        r = state_pickler.get_state(a)
        self.assertEqual(r.__metadata__['has_instance'], True)
        l = [1, a]
        r = state_pickler.get_state(l)
        self.assertEqual(r.has_instance, True)
        self.assertEqual(r[1].__metadata__['has_instance'], True)
        d = {'a': l, 'b': 1}
        r = state_pickler.get_state(d)
        self.assertEqual(r.has_instance, True)
        self.assertEqual(r['a'].has_instance, True)
        self.assertEqual(r['a'][1].__metadata__['has_instance'], True)

        class B:

            def __init__(self):
                self.a = [1, A()]

        b = B()
        r = state_pickler.get_state(b)
        self.assertEqual(r.__metadata__['has_instance'], True)
        self.assertEqual(r.a.has_instance, True)
        self.assertEqual(r.a[1].__metadata__['has_instance'], True)

    def test_pickle_classic(self):
        """Test if classic classes can be pickled."""
        t = TestClassic()
        self.set_object(t)
        # Generate the dict that is actually pickled.
        state = state_pickler.StatePickler().dump_state(t)

        # First check if all the attributes are handled.
        keys = sorted(state['data']['data'].keys())
        expect = [x for x in t.__dict__.keys() if '__' not in x]
        expect.sort()
        self.assertEqual(keys, expect)
        # Check each attribute.
        self.verify(t, state)

    def test_unpickle_classic(self):
        """Test if classic classes can be unpickled."""
        t = TestClassic()
        self.set_object(t)
        # Get the pickled state.
        res = state_pickler.get_state(t)
        # Check each attribute.
        self.verify_unpickled(t, res)

    def test_state_setter_classic(self):
        """Test if classic classes' state can be set."""
        t = TestClassic()
        self.set_object(t)
        # Get the pickled state.
        res = state_pickler.get_state(t)

        # Now create a new instance and set its state.
        t1 = state_pickler.create_instance(res)
        state_pickler.set_state(t1, res)
        # Check each attribute.
        self.verify_unpickled(t1, res)

    def test_state_setter(self):
        """Test some of the features of the set_state method."""
        t = TestClassic()
        self.set_object(t)
        # Get the saved state.
        res = state_pickler.get_state(t)

        # Now create a new instance and test the setter.
        t1 = state_pickler.create_instance(res)

        keys = ['c', 'b', 'f', 'i', 'tuple', 'list', 'l', 'numeric',
                'n', 's', 'u', 'pure_list', 'inst', 'ref', 'dict']
        ignore = list(keys)
        ignore.remove('b')
        first = ['b']
        last = []
        state_pickler.set_state(t1, res, ignore=ignore, first=first, last=last)
        # Only 'b' should have been set.
        self.assertEqual(t1.b, True)
        # Rest are unchanged.
        self.assertEqual(t1.i, 7)
        self.assertEqual(t1.s, 'String')
        self.assertEqual(t1.u, u'Unicode')
        self.assertEqual(t1.inst.a, 'a')
        self.assertEqual(t1.list[0], 1)
        self.assertEqual(t1.tuple[-1].a, 'a')
        self.assertEqual(t1.dict['a'], 1)

        # Check if last works.
        last = ignore
        ignore = []
        first = []
        state_pickler.set_state(t1, res, ignore=ignore, first=first, last=last)
        # Check everything.
        self.verify_unpickled(t1, res)

    def test_pickle_traits(self):
        """Test if traited classes can be pickled."""
        t = TestTraits()
        self.set_object(t)

        # Generate the dict that is actually pickled.
        state = state_pickler.StatePickler().dump_state(t)

        # First check if all the attributes are handled.
        keys = sorted(state['data']['data'].keys())
        expect = [x for x in t.__dict__.keys() if '__' not in x]
        expect.sort()
        self.assertEqual(keys, expect)
        # Check each attribute.
        self.verify(t, state)

    def test_unpickle_traits(self):
        """Test if traited classes can be unpickled."""
        t = TestTraits()
        self.set_object(t)

        # Get the pickled state.
        res = state_pickler.get_state(t)
        # Check each attribute.
        self.verify_unpickled(t, res)

    def test_state_setter_traits(self):
        """Test if traited classes' state can be set."""
        t = TestTraits()
        self.set_object(t)

        # Get the saved state.
        res = state_pickler.get_state(t)

        # Now create a new instance and set its state.
        t1 = state_pickler.create_instance(res)
        state_pickler.set_state(t1, res)
        # Check each attribute.
        self.verify_unpickled(t1, res)

    def test_reference_cycle(self):
        """Test if reference cycles are handled when setting the state."""
        class A:
            pass

        class B:
            pass
        a = A()
        b = B()
        a.a = b
        b.b = a
        state = state_pickler.get_state(a)
        z = A()
        z.a = B()
        z.a.b = z
        state_pickler.set_state(z, state)

    def test_get_state_on_tuple_with_numeric_references(self):
        num = numpy.zeros(10, float)
        data = (num, num)
        # If this just completes without error, we are good.
        state = state_pickler.get_state(data)
        # The two should be the same object.
        self.assertTrue(state[0] is state[1])
        numpy.testing.assert_allclose(state[0], num)

    def test_state_is_saveable(self):
        """Test if the state can be saved like the object itself."""
        t = TestClassic()
        self.set_object(t)
        state = state_pickler.get_state(t)
        # Now get the state of the state itself.
        state1 = state_pickler.get_state(state)
        self.verify_state(state1, state)

        # Same thing for the traited class.
        t = TestTraits()
        self.set_object(t)
        state = state_pickler.get_state(t)
        # Now get the state of the state itself.
        state1 = state_pickler.get_state(state)
        self.verify_state(state1, state)

    def test_get_pure_state(self):
        """Test if get_pure_state is called first."""
        class B:

            def __init__(self):
                self.a = 'dict'

            def __get_pure_state__(self):
                return {'a': 'get_pure_state'}

            def __getstate__(self):
                return {'a': 'getstate'}
        b = B()
        s = state_pickler.get_state(b)
        self.assertEqual(s.a, 'get_pure_state')
        del B.__get_pure_state__
        s = state_pickler.get_state(b)
        self.assertEqual(s.a, 'getstate')
        del B.__getstate__
        s = state_pickler.get_state(b)
        self.assertEqual(s.a, 'dict')


if __name__ == "__main__":
    unittest.main()



#------------------------------------------------------------------------------
# Copyright (c) 2008, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought permissions package component>
#------------------------------------------------------------------------------


# Enthought library imports.
from pyface.action.api import Action
from traits.api import Unicode

# Local imports.
from apptools.permissions.package_globals import get_permissions_manager


class LoginAction(Action):
    """An action that authenticates the current user."""

    #### 'Action' interface ###################################################

    name = Unicode("Log&in...")

    ###########################################################################
    # 'Action' interface.
    ###########################################################################

    def perform(self, event):
        """Perform the action."""

        get_permissions_manager().user_manager.authenticate_user()



#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought pyface package component>
#------------------------------------------------------------------------------
""" A log handler that allows filtering of messages by origin. """


# Standard library imports.
import logging, inspect, os

# Local imports.
#
# fixme: This module was just copied over from 'envisage.core' (so
# that we don't rely on Envisage here!). Where should this module go?
from .util import get_module_name


class FilteringHandler(logging.Handler):
    """ A log handler that allows filtering of messages by origin.

    Example
    -------
    ::

        from apptools.logger.api import DebugHandler, logger

        handler = FilteringHandler(
            include = {
                'envisage.core' : True
            },

            exclude = {
                'envisage.core.application' : False
            }
        )

        logger.addHandler(handler)

    Notes
    -----

    The boolean value specified for each name in the include and exclude
    dictionaries indicates whether or not the include or exclude should pertain
    to any sub-packages and modules.

    The above example includes all log messages from anything contained in, or
    under the 'envisage.core' package, EXCEPT for any log messages
    from the 'envisage.core.application' module.

    """

    ###########################################################################
    # 'object' interface.
    ###########################################################################

    def __init__(self, include=None, exclude=None):
        """ Creates a new handler. """

        # Base class constructor.
        logging.Handler.__init__(self)

        # Packages or modules to include.
        self.include = include is not None and include or {}

        # Packages or modules to exclude.
        self.exclude = exclude is not None and exclude or {}

        return

    ###########################################################################
    # 'Handler' interface.
    ###########################################################################

    def emit(self, record):
        """ Emits a log record. """

        # Get the name of the module that the logger was called from.
        module_name = self._get_module_name()

        if len(self.include) == 0 or self._include(module_name):
            if len(self.exclude) == 0 or not self._exclude(module_name):
                self.filtered_emit(record)

        return

    ###########################################################################
    # 'Handler' interface.
    ###########################################################################

    def filtered_emit(self, record):
        """ Emits a log record if it has not been filtered. """

        print(record.getMessage())

        return

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _get_module_name(self):
        """ Returns the module that the logger was actually called from. """

        # fixem: Ahem.... what can I say... this gets us to the actual caller!
        frame = inspect.currentframe()
        frame = frame.f_back.f_back.f_back.f_back.f_back.f_back.f_back

        # This returns a tuple containing the last 5 elements of the frame
        # record which are:-
        #
        # - the filename
        # - the line number
        # - the function name
        # - the list of lines of context from the source code
        # - the index of the current line within that list
        filename, lineno, funcname, source, index = inspect.getframeinfo(frame)

        # The plugin definition's location is the directory containing the
        # module that it is defined in.
        self.location = os.path.dirname(filename)

        # We can't use 'inspect.getmodulename' here as it gets confused because
        # of our import hook in Envisage 8^(
        #
        # e.g. for the core plugin definition:-
        #
        # using inspect -> 'core_plugin_definition'
        # using ours    -> 'envisage.core.core_plugin_definition'
        return get_module_name(filename)

    def _include(self, module_name):
        """ Is the module name in the include set? """

        for item, include_children in list(self.include.items()):
            if item == module_name:
                include = True
                break

            elif include_children and self._is_child_of(item, module_name):
                include = True
                break

        else:
            include = False

        return include

    def _exclude(self, module_name):
        """ Is the module name in the exclude set? """

        for item, exclude_children in list(self.exclude.items()):
            if item == module_name:
                exclude = True
                break

            elif exclude_children and self._is_child_of(item, module_name):
                exclude = True
                break

        else:
            exclude = False

        return exclude

    def _is_child_of(self, x, y):
        """ Is 'y' a child symbol of 'x'?

        e.g.

        'foo.bar.baz' (y) is a child of 'foo.bar' (x)

        """

        if y.startswith(x):
            x_atoms = x.split('.')
            y_atoms = y.split('.')

            is_child_of = y_atoms[:len(x_atoms)] == x_atoms

        else:
            is_child_of = False

        return is_child_of

#### EOF ######################################################################



#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought naming package component>
#------------------------------------------------------------------------------
""" State factory for Python File System contexts. """


# Enthought library imports.
from apptools.io.api import File

# Local imports.
from .address import Address
from .reference import Reference
from .state_factory import StateFactory


class PyFSStateFactory(StateFactory):
    """ State factory for Python File System contexts. """

    ###########################################################################
    # 'StateFactory' interface.
    ###########################################################################

    def get_state_to_bind(self, obj, name, context):
        """ Returns the state of an object for binding. """

        state = None

        if isinstance(obj, File):
            # If the file is not actually in the directory represented by the
            # context then we create and bind a reference to it.
            if obj.parent.path != context.path:
                state = Reference(
                    class_name = obj.__class__.__name__,
                    addresses  = [Address(type='file', content=obj.path)]
                )

        return state

### EOF #######################################################################



#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought naming package component>
#------------------------------------------------------------------------------
""" The base class for all naming contexts. """


# Enthought library imports.
from traits.api import Any, Dict, Event, HasTraits, Instance
from traits.api import Property, Str
from apptools.type_manager.api import TypeManager

# Local imports.
from .binding import Binding
from .exception import InvalidNameError, NameAlreadyBoundError
from .exception import NameNotFoundError, NotContextError
from .exception import OperationNotSupportedError
from .naming_event import NamingEvent
from .naming_manager import naming_manager
from .object_factory import ObjectFactory
from .state_factory import StateFactory
from .unique_name import make_unique_name


# Constants for environment property keys.
INITIAL_CONTEXT_FACTORY = "apptools.naming.factory.initial"
OBJECT_FACTORIES = "apptools.naming.factory.object"
STATE_FACTORIES = "apptools.naming.factory.state"

# Non-JNDI.
TYPE_MANAGER = "apptools.naming.factory.type.manager"


# The default environment.
ENVIRONMENT = {
    # 'Context' properties.
    OBJECT_FACTORIES          : [],
    STATE_FACTORIES           : [],

    # Non-JNDI.
    TYPE_MANAGER              : None,
}


class Context(HasTraits):
    """ The base class for all naming contexts. """

    # Keys for environment properties.
    INITIAL_CONTEXT_FACTORY = INITIAL_CONTEXT_FACTORY
    OBJECT_FACTORIES = OBJECT_FACTORIES
    STATE_FACTORIES = STATE_FACTORIES

    # Non-JNDI.
    TYPE_MANAGER = TYPE_MANAGER

    #### 'Context' interface ##################################################

    # The naming environment in effect for this context.
    environment = Dict(ENVIRONMENT)

    # The name of the context within its own namespace.
    namespace_name = Property(Str)

    # The type manager in the context's environment (used to create context
    # adapters etc.).
    #
    # fixme: This is an experimental 'convenience' trait, since it is common
    # to get hold of the context's type manager to see if some object has a
    # context adapter.
    type_manager = Property(Instance(TypeManager))

    #### Events ####

    # Fired when an object has been added to the context (either via 'bind' or
    # 'create_subcontext').
    object_added = Event(NamingEvent)

    # Fired when an object has been changed (via 'rebind').
    object_changed = Event(NamingEvent)

    # Fired when an object has been removed from the context (either via
    # 'unbind' or 'destroy_subcontext').
    object_removed = Event(NamingEvent)

    # Fired when an object in the context has been renamed (via 'rename').
    object_renamed = Event(NamingEvent)

    # Fired when the contents of the context have changed dramatically.
    context_changed = Event(NamingEvent)

     #### Protected 'Context' interface #######################################

    # The bindings in the context.
    _bindings = Dict(Str, Any)

    ###########################################################################
    # 'Context' interface.
    ###########################################################################

    #### Properties ###########################################################

    def _get_namespace_name(self):
        """
        Return the name of the context within its own namespace.

        That is the full-path, through the namespace this context participates
        in, to get to this context.  For example, if the root context of the
        namespace was called 'Foo', and there was a subcontext of that called
        'Bar', and we were within that and called 'Baz', then this should
        return 'Foo/Bar/Baz'.

        """

        # FIXME: We'd like to raise an exception and force implementors to
        # decide what to do.  However, it appears to be pretty common that
        # most Context implementations do not override this method -- possibly
        # because the comments aren't clear on what this is supposed to be?
        #
        # Anyway, if we raise an exception then it is impossible to use any
        # evaluations when building a Traits UI for a Context.  That is, the
        # Traits UI can't include items that have a 'visible_when' or
        # 'enabled_when' evaluation.  This is because the Traits evaluation
        # code calls the 'get()' method on the Context which attempts to
        # retrieve the current namespace_name value.
        #raise OperationNotSupportedError()
        return ''

    def _get_type_manager(self):
        """ Returns the type manager in the context's environment.

        This will return None if no type manager was used to create the initial
        context.

        """

        return self.environment.get(self.TYPE_MANAGER)

    #### Methods ##############################################################

    def bind(self, name, obj, make_contexts=False):
        """ Binds a name to an object.

        If 'make_contexts' is True then any missing intermediate contexts are
        created automatically.

        """

        if len(name) == 0:
            raise InvalidNameError('empty name')

        # Parse the name.
        components = self._parse_name(name)

        # If there is exactly one component in the name then the operation
        # takes place in this context.
        if len(components) == 1:
            atom = components[0]

            # Is the name already bound?
            if self._is_bound(atom):
                raise NameAlreadyBoundError(name)

            # Do the actual bind.
            self._bind(atom, obj)

            # Trait event notification.
            self.object_added = NamingEvent(
                new_binding=Binding(name=name, obj=obj, context=self)
            )

        # Otherwise, attempt to continue resolution into the next context.
        else:
            if not self._is_bound(components[0]):
                if make_contexts:
                    self._create_subcontext(components[0])

                else:
                    raise NameNotFoundError(components[0])

            next_context = self._get_next_context(components[0])
            next_context.bind('/'.join(components[1:]), obj, make_contexts)

        return

    def rebind(self, name, obj, make_contexts=False):
        """ Binds an object to a name that may already be bound.

        If 'make_contexts' is True then any missing intermediate contexts are
        created automatically.

        The object may be a different object but may also be the same object
        that is already bound to the specified name. The name may or may not be
        already used. Think of this as a safer version of 'bind' since this
        one will never raise an exception regarding a name being used.

        """

        if len(name) == 0:
            raise InvalidNameError('empty name')

        # Parse the name.
        components = self._parse_name(name)

        # If there is exactly one component in the name then the operation
        # takes place in this context.
        if len(components) == 1:
            # Do the actual rebind.
            self._rebind(components[0], obj)

            # Trait event notification.
            self.object_changed = NamingEvent(
                new_binding=Binding(name=name, obj=obj, context=self)
            )

        # Otherwise, attempt to continue resolution into the next context.
        else:
            if not self._is_bound(components[0]):
                if make_contexts:
                    self._create_subcontext(components[0])

                else:
                    raise NameNotFoundError(components[0])

            next_context = self._get_next_context(components[0])
            next_context.rebind('/'.join(components[1:]), obj, make_contexts)

        return

    def unbind(self, name):
        """ Unbinds a name. """

        if len(name) == 0:
            raise InvalidNameError('empty name')

        # Parse the name.
        components = self._parse_name(name)

        # If there is exactly one component in the name then the operation
        # takes place in this context.
        if len(components) == 1:
            atom = components[0]

            if not self._is_bound(atom):
                raise NameNotFoundError(name)

            # Lookup the object that we are unbinding to use in the event
            # notification.
            obj = self._lookup(atom)

            # Do the actual unbind.
            self._unbind(atom)

            # Trait event notification.
            self.object_removed = NamingEvent(
                old_binding=Binding(name=name, obj=obj, context=self)
            )

        # Otherwise, attempt to continue resolution into the next context.
        else:
            if not self._is_bound(components[0]):
                raise NameNotFoundError(components[0])

            next_context = self._get_next_context(components[0])
            next_context.unbind('/'.join(components[1:]))

        return

    def rename(self, old_name, new_name):
        """ Binds a new name to an object. """

        if len(old_name) == 0 or len(new_name) == 0:
            raise InvalidNameError('empty name')

        # Parse the names.
        old_components = self._parse_name(old_name)
        new_components = self._parse_name(new_name)

        # If there is axactly one component in BOTH names then the operation
        # takes place ENTIRELY in this context.
        if len(old_components) == 1 and len(new_components) == 1:
            # Is the old name actually bound?
            if not self._is_bound(old_name):
                raise NameNotFoundError(old_name)

            # Is the new name already bound?
            if self._is_bound(new_name):
                raise NameAlreadyBoundError(new_name)

            # Do the actual rename.
            self._rename(old_name, new_name)

            # Lookup the object that we are renaming to use in the event
            # notification.
            obj = self._lookup(new_name)

            # Trait event notification.
            self.object_renamed = NamingEvent(
                old_binding=Binding(name=old_name, obj=obj, context=self),
                new_binding=Binding(name=new_name, obj=obj, context=self)
            )

        else:
            # fixme: This really needs to be transactional in case the bind
            # succeeds but the unbind fails.  To be safe should we just not
            # support cross-context renaming for now?!?!
            #
            # Lookup the object.
            obj = self.lookup(old_name)

            # Bind the new name.
            self.bind(new_name, obj)

            # Unbind the old one.
            self.unbind(old_name)

        return

    def lookup(self, name):
        """ Resolves a name relative to this context. """

        # If the name is empty we return the context itself.
        if len(name) == 0:
            # fixme: The JNDI spec. says that this should return a COPY of
            # the context.
            return self

        # Parse the name.
        components = self._parse_name(name)

        # If there is exactly one component in the name then the operation
        # takes place in this context.
        if len(components) == 1:
            atom = components[0]

            if not self._is_bound(atom):
                raise NameNotFoundError(name)

            # Do the actual lookup.
            obj = self._lookup(atom)

        # Otherwise, attempt to continue resolution into the next context.
        else:
            if not self._is_bound(components[0]):
                raise NameNotFoundError(components[0])

            next_context = self._get_next_context(components[0])
            obj = next_context.lookup('/'.join(components[1:]))

        return obj


    # fixme: Non-JNDI
    def lookup_binding(self, name):
        """ Looks up the binding for a name relative to this context. """

        if len(name) == 0:
            raise InvalidNameError('empty name')

        # Parse the name.
        components = self._parse_name(name)

        # If there is exactly one component in the name then the operation
        # takes place in this context.
        if len(components) == 1:
            atom = components[0]

            if not self._is_bound(atom):
                raise NameNotFoundError(name)

            # Do the actual lookup.
            binding = self._lookup_binding(atom)

        # Otherwise, attempt to continue resolution into the next context.
        else:
            if not self._is_bound(components[0]):
                raise NameNotFoundError(components[0])

            next_context = self._get_next_context(components[0])
            binding = next_context.lookup_binding('/'.join(components[1:]))

        return binding

    # fixme: Non-JNDI
    def lookup_context(self, name):
        """ Resolves a name relative to this context.

        The name MUST resolve to a context. This method is useful to return
        context adapters.

        """

        # If the name is empty we return the context itself.
        if len(name) == 0:
            # fixme: The JNDI spec. says that this should return a COPY of
            # the context.
            return self

        # Parse the name.
        components = self._parse_name(name)

        # If there is exactly one component in the name then the operation
        # takes place in this context.
        if len(components) == 1:
            atom = components[0]

            if not self._is_bound(atom):
                raise NameNotFoundError(name)

            # Do the actual lookup.
            obj = self._get_next_context(atom)

        # Otherwise, attempt to continue resolution into the next context.
        else:
            if not self._is_bound(components[0]):
                raise NameNotFoundError(components[0])

            next_context = self._get_next_context(components[0])
            obj = next_context.lookup('/'.join(components[1:]))

        return obj

    def create_subcontext(self, name):
        """ Creates a sub-context. """

        if len(name) == 0:
            raise InvalidNameError('empty name')

        # Parse the name.
        components = self._parse_name(name)

        # If there is exactly one component in the name then the operation
        # takes place in this context.
        if len(components) == 1:
            atom = components[0]

            # Is the name already bound?
            if self._is_bound(atom):
                raise NameAlreadyBoundError(name)

            # Do the actual creation of the sub-context.
            sub = self._create_subcontext(atom)

            # Trait event notification.
            self.object_added = NamingEvent(
                new_binding=Binding(name=name, obj=sub, context=self)
            )

        # Otherwise, attempt to continue resolution into the next context.
        else:
            if not self._is_bound(components[0]):
                raise NameNotFoundError(components[0])

            next_context = self._get_next_context(components[0])
            sub = next_context.create_subcontext('/'.join(components[1:]))

        return sub

    def destroy_subcontext(self, name):
        """ Destroys a sub-context. """

        if len(name) == 0:
            raise InvalidNameError('empty name')

        # Parse the name.
        components = self._parse_name(name)

        # If there is exactly one component in the name then the operation
        # takes place in this context.
        if len(components) == 1:
            atom = components[0]

            if not self._is_bound(atom):
                raise NameNotFoundError(name)

            obj = self._lookup(atom)
            if not self._is_context(atom):
               raise NotContextError(name)

            # Do the actual destruction of the sub-context.
            self._destroy_subcontext(atom)

            # Trait event notification.
            self.object_removed = NamingEvent(
                old_binding=Binding(name=name, obj=obj, context=self)
            )

        # Otherwise, attempt to continue resolution into the next context.
        else:
            if not self._is_bound(components[0]):
                raise NameNotFoundError(components[0])

            next_context = self._get_next_context(components[0])
            next_context.destroy_subcontext('/'.join(components[1:]))

        return

    # fixme: Non-JNDI
    def get_unique_name(self, prefix):
        """ Returns a name that is unique within the context.

        The name returned will start with the specified prefix.

        """

        return make_unique_name(prefix, existing=self.list_names(''),
            format='%s (%d)')


    def list_names(self, name=''):
        """ Lists the names bound in a context. """

        # If the name is empty then the operation takes place in this context.
        if len(name) == 0:
            names = self._list_names()

        # Otherwise, attempt to continue resolution into the next context.
        else:
            # Parse the name.
            components = self._parse_name(name)

            if not self._is_bound(components[0]):
                raise NameNotFoundError(components[0])

            next_context = self._get_next_context(components[0])
            names = next_context.list_names('/'.join(components[1:]))

        return names

    def list_bindings(self, name=''):
        """ Lists the bindings in a context. """

        # If the name is empty then the operation takes place in this context.
        if len(name) == 0:
            bindings = self._list_bindings()

        # Otherwise, attempt to continue resolution into the next context.
        else:
            # Parse the name.
            components = self._parse_name(name)

            if not self._is_bound(components[0]):
                raise NameNotFoundError(components[0])

            next_context = self._get_next_context(components[0])
            bindings = next_context.list_bindings('/'.join(components[1:]))

        return bindings

    # fixme: Non-JNDI
    def is_context(self, name):
        """ Returns True if the name is bound to a context. """

        # If the name is empty then it refers to this context.
        if len(name) == 0:
            is_context = True

        else:
            # Parse the name.
            components = self._parse_name(name)

            # If there is exactly one component in the name then the operation
            # takes place in this context.
            if len(components) == 1:
                atom = components[0]

                if not self._is_bound(atom):
                    raise NameNotFoundError(name)

                # Do the actual check.
                is_context = self._is_context(atom)

            # Otherwise, attempt to continue resolution into the next context.
            else:
                if not self._is_bound(components[0]):
                    raise NameNotFoundError(components[0])

                next_context = self._get_next_context(components[0])
                is_context = next_context.is_context('/'.join(components[1:]))

        return is_context

    # fixme: Non-JNDI
    def search(self, obj):
        """ Returns a list of namespace names that are bound to obj. """

        # don't look for None
        if obj is None:
            return []

        # Obj is bound to these names relative to this context
        names = []

        # path contain the name components down to the current context
        path = []

        self._search( obj, names, path, {} )

        return names

    ###########################################################################
    # Protected 'Context' interface.
    ###########################################################################

    def _parse_name(self, name):
        """ Parse a name into a list of components.

        e.g. 'foo/bar/baz' -> ['foo', 'bar', 'baz']

        """

        return name.split('/')

    def _is_bound(self, name):
        """ Is a name bound in this context? """

        return name in self._bindings

    def _lookup(self, name):
        """ Looks up a name in this context. """

        obj = self._bindings[name]

        return naming_manager.get_object_instance(obj, name, self)

    def _lookup_binding(self, name):
        """ Looks up the binding for a name in this context. """

        return Binding(name=name, obj=self._lookup(name), context=self)

    def _bind(self, name, obj):
        """ Binds a name to an object in this context. """

        state = naming_manager.get_state_to_bind(obj, name, self)
        self._bindings[name] = state

        return

    def _rebind(self, name, obj):
        """ Rebinds a name to an object in this context. """

        self._bind(name, obj)

        return

    def _unbind(self, name):
        """ Unbinds a name from this context. """

        del self._bindings[name]

        return

    def _rename(self, old_name, new_name):
        """ Renames an object in this context. """

        # Bind the new name.
        self._bindings[new_name] = self._bindings[old_name]

        # Unbind the old one.
        del self._bindings[old_name]

        return

    def _create_subcontext(self, name):
        """ Creates a sub-context of this context. """

        sub = self.__class__(environment=self.environment)
        self._bindings[name] = sub

        return sub

    def _destroy_subcontext(self, name):
        """ Destroys a sub-context of this context. """

        del self._bindings[name]

        return

    def _list_bindings(self):
        """ Lists the bindings in this context. """

        bindings = []
        for name in self._list_names():
            bindings.append(
                Binding(name=name, obj=self._lookup(name), context=self)
            )

        return bindings

    def _list_names(self):
        """ Lists the names bound in this context. """

        return list(self._bindings.keys())

    def _is_context(self, name):
        """ Returns True if a name is bound to a context. """

        return self._get_next_context(name) is not None

    def _get_next_context(self, name):
        """ Returns the next context. """

        obj = self._lookup(name)

        # If the object is a context then everything is just dandy.
        if isinstance(obj, Context):
            next_context = obj

        # Otherwise, instead of just giving up, see if the context has a type
        # manager that knows how to adapt the object to make it quack like a
        # context.
        else:
            next_context = self._get_context_adapter(obj)

            # If no adapter was found then we cannot continue name resolution.
            if next_context is None:
                raise NotContextError(name)

        return next_context

    def _search( self, obj, names, path, searched):
        """ Append to names any name bound to obj.
            Join path and name with '/' to for a complete name from the
            top context.
        """

        # Check the bindings recursively.
        for binding in self.list_bindings():
            if binding.obj is obj:
                path.append( binding.name )
                names.append( '/'.join(path) )
                path.pop()

            if isinstance( binding.obj, Context ) \
                and not binding.obj in searched:
                path.append( binding.name )
                searched[binding.obj] = True
                binding.obj._search( obj, names, path, searched )
                path.pop()

        return

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _get_context_adapter(self, obj):
        """ Returns a context adapter for an object.

        Returns None if no such adapter is available.

        """

        if self.type_manager is not None:
            adapter = self.type_manager.object_as(
                obj, Context, environment=self.environment, context=self
            )

        else:
            adapter = None

        return adapter

#### EOF ######################################################################



""" Package-scope globals.

The default preferences node is currently used by 'PreferencesHelper' and
'PreferencesBinding' instances if no specific preferences node is set. This
makes it easy for them to access the root node of an application-wide
preferences hierarchy.

"""


# The default preferences node.
_default_preferences = None

def get_default_preferences():
    """ Get the default preferences node. """

    return _default_preferences

def set_default_preferences(default_preferences):
    """ Set the default preferences node. """

    global _default_preferences

    _default_preferences = default_preferences

    # For convenience.
    return _default_preferences

#### EOF ######################################################################



from .i_preferences import IPreferences

from .package_globals import get_default_preferences, set_default_preferences
from .preferences import Preferences
from .preference_binding import PreferenceBinding, bind_preference
from .preferences_helper import PreferencesHelper
from .scoped_preferences import ScopedPreferences



""" Tests the type manager. """


# Standard library imports.
import unittest

# Enthought library imports.
from traits.api import HasTraits, Instance, Str
from apptools.type_manager import AdapterFactory, Factory, TypeManager


# Test classes.
class Foo(HasTraits):
    name = Str

    def foogle(self):
        return 'Foo.foogle.%s' % self.name


class SubOfFoo(Foo):
    def foogle(self):
        return 'Sub.foogle.%s' % self.name

class EmptySubOfFoo(Foo):
    pass

class Bar(HasTraits):
    name = Str
    def blargle(self):
        return 'Bar.blargle.%s' % self.name


# Test adapters and factories.
class FooToBarAdapter(HasTraits):
    adaptee = Instance(Foo)

    def blargle(self):
        return self.adaptee.foogle()

class FooToBarAdapterFactory(AdapterFactory):

    #### 'AdapterFactory' interface ###########################################

    # The type of object that the factory can adapt.
    adaptee_class = Foo

    # The adapter class (the class that adapts the adaptee to the target
    # class).
    adapter_class = FooToBarAdapter

    # The target class (the class that the factory can adapt objects to).
    target_class = Bar


class SubOfFooToBarAdapter(HasTraits):
    adaptee = Instance(SubOfFoo)

    def blargle(self):
        return self.adaptee.foogle()

class SubOfFooToBarAdapterFactory(AdapterFactory):
    #### 'AdapterFactory' interface ###########################################

    # The type of object that the factory can adapt.
    adaptee_class = SubOfFoo

    # The adapter class (the class that adapts the adaptee to the target
    # class).
    adapter_class = SubOfFooToBarAdapter

    # The target class (the class that the factory can adapt objects to).
    target_class = Bar


class BarFactory(Factory):
    def can_create(self, target_class, *args, **kw):
        return target_class is Bar

    def create(self, target_class, *args, **kw):
        return Bar(*args, **kw)


class TypeManagerTestCase(unittest.TestCase):
    """ Tests the type manager. """

    ###########################################################################
    # 'TestCase' interface.
    ###########################################################################

    def setUp(self):
        """ Prepares the test fixture before each test method is called. """

        # An empty type manager.
        self.type_manager = TypeManager()

        return

    def tearDown(self):
        """ Called immediately after each test method has been called. """

        return

    ###########################################################################
    # Tests.
    ###########################################################################

    def test_no_adapter_required(self):
        """ no adapter required """

        # Create a Bar.
        b = Bar()

        # Try to adapt it to a Bar.
        bar = self.type_manager.object_as(b, Bar)

        # The type manager should simply return the same object.
        self.assert_(bar is b)

        return

    def test_no_adapter(self):
        """ no adapter available """

        # Create a Foo.
        foo = Foo(name='fred')

        # Try to adapt it to a bar.
        bar = self.type_manager.object_as(foo, Bar)

        # There should be no way to adapt a Foo to a Bar.
        self.assertEqual(bar, None)

        return

    def test_instance_adapter(self):
        """ instance adapter """

        # Create a Foo.
        foo = Foo(name='fred')

        # Register an adapter Foo->Bar on the INSTANCE (this one should take
        # precedence).
        self.type_manager.register_instance_adapters(
            FooToBarAdapterFactory(), foo
        )

        # Register an adapter Foo->Bar on the TYPE (this will fail if it gets
        # picked up since it won't actually adapt 'Foo' objects!).
        self.type_manager.register_instance_adapters(
            SubOfFooToBarAdapterFactory(), Foo
        )

        # Adapt it to a Bar.
        bar = self.type_manager.object_as(foo, Bar)
        self.assertNotEqual(bar, None)
        self.assertEqual(bar.blargle(), 'Foo.foogle.fred')

        return

    def test_unregister_instance_adapter(self):
        """ unregister instance adapter """

        # Create a Foo.
        foo = Foo(name='fred')

        # The factory.
        factory = FooToBarAdapterFactory()

        # Register an adapter Foo->Bar on the INSTANCE (this one should take
        # precedence).
        self.type_manager.register_instance_adapters(factory, foo)

        # Adapt it to a Bar.
        bar = self.type_manager.object_as(foo, Bar)
        self.assertNotEqual(bar, None)
        self.assertEqual(bar.blargle(), 'Foo.foogle.fred')

        # Remove the adapter.
        self.type_manager.unregister_instance_adapters(factory, foo)

        # Now we shouldn't be able to adapt the object.
        #
        # Try to adapt it to a bar.
        bar = self.type_manager.object_as(foo, Bar)

        # There should be no way to adapt a Foo to a Bar.
        self.assertEqual(bar, None)

        return

    def test_adapter_on_class(self):
        """ an adapter registered on an object's actual class. """

        # Register an adapter Foo->Bar.
        self.type_manager.register_type_adapters(FooToBarAdapterFactory(), Foo)

        # Create a Foo.
        foo = Foo(name='fred')

        # Adapt it to a Bar.
        bar = self.type_manager.object_as(foo, Bar)
        self.assertNotEqual(bar, None)
        self.assertEqual(bar.blargle(), 'Foo.foogle.fred')

        return

    def test_adapter_on_base_class(self):
        """ an adapter registered on one of an object's base classes. """

        # Register an adapter Foo->Bar.
        self.type_manager.register_type_adapters(FooToBarAdapterFactory(), Foo)

        # Create an instance of a class derived from Foo.
        sub = SubOfFoo(name='fred')

        # Adapt it to a Bar.
        bar = self.type_manager.object_as(sub, Bar)
        self.assertNotEqual(bar, None)
        self.assertEqual(bar.blargle(), 'Sub.foogle.fred')

        return

    def test_ignore_adapter_on_class(self):
        """ ignore an adapter on an object's actual class. """

        # Register an adapter SubOfFoo->Bar on the Foo class.
        self.type_manager.register_type_adapters(
            SubOfFooToBarAdapterFactory(), Foo
        )

        # Create a Foo.
        foo = Foo(name='fred')

        # Adapt it to a Bar.
        bar = self.type_manager.object_as(foo, Bar)

        # There should be no way to adapt a Foo to a Bar.
        self.assertEqual(bar, None)

        return

    def test_ignore_adapter_on_derived_class(self):
        """ ignore an adapter registered on a derived class. """

        # Register an adapter Foo->Bar on the SubOfFoo class.
        self.type_manager.register_type_adapters(
            FooToBarAdapterFactory(), SubOfFoo
        )

        # Create a Foo.
        foo = Foo(name='fred')

        # Adapt it to a Bar.
        bar = self.type_manager.object_as(foo, Bar)

        # There should be no way to adapt a Foo to a Bar.
        self.assertEqual(bar, None)

        return

    def test_unregister_adapter(self):
        """ unregister an adapter. """

        factory = FooToBarAdapterFactory()

        # Register an adapter Foo->Bar.
        self.type_manager.register_type_adapters(factory, Foo)

        # Create a Foo.
        foo = Foo(name='fred')

        # Adapt it to a Bar.
        bar = self.type_manager.object_as(foo, Bar)
        self.assertNotEqual(bar, None)
        self.assertEqual(bar.blargle(), 'Foo.foogle.fred')

        # Unregister the adapter.
        self.type_manager.unregister_type_adapters(factory)

        # Try to adapt it to a Bar.
        bar = self.type_manager.object_as(foo, Bar)

        # There should be no way to adapt a Foo to a Bar.
        self.assertEqual(bar, None)

        return

    def test_factory(self):
        """ simple factory """

        # Create a Bar factory.
        factory = BarFactory()

        # Try to create a Bar using the factory.
        bar = self.type_manager.object_as(factory, Bar, name='joe')
        self.assertNotEqual(bar, None)
        self.assertEqual(bar.blargle(), 'Bar.blargle.joe')

        return

    def test_pre_hook(self):
        """ pre hook """

        l = []
        def hook(*args, **kw):
            l.append('Hello')

        # Create a Foo.
        foo = Foo(name='fred')

        # Hook a method.
        self.type_manager.add_pre(Foo, 'foogle', hook)

        # Call the method that we have hooked.
        foo.foogle()

        # Make sure that the hook was called.
        self.assertEqual(len(l), 1)

        # Remove the hook.
        self.type_manager.remove_pre(Foo, 'foogle', hook)

        # Call the method that we have hooked.
        foo.foogle()

        # Make sure that the hook was NOT called.
        self.assertEqual(len(l), 1)

        return

    def test_post_hook(self):
        """ post hook """

        def hook(result, *args, **kw):
            return 'Hello'

        # Create a Foo.
        foo = Foo(name='fred')

        # Hook a method.
        self.type_manager.add_post(Foo, 'foogle', hook)

        # Call the method that we have hooked.
        self.assertEqual(foo.foogle(), 'Hello')

        # Remove the hook.
        self.type_manager.remove_post(Foo, 'foogle', hook)

        # Call the method that we have hooked.
        foo.foogle()

        # Make sure that the hook was NOT called.
        self.assertEqual(foo.foogle(), 'Foo.foogle.fred')

        return

    def test_pre_hook_on_inherited_method(self):
        """ test pre hook on an inherited method """

        l = []
        def hook(*args, **kw):
            l.append('Hello')

        # Create an instance of a subclass of Foo that does NOT override
        # 'foogle'.
        esof = EmptySubOfFoo(name='fred')

        # Prove that it does not override 'foogle'!
        method = EmptySubOfFoo.__dict__.get('foogle')
        self.assertEqual(method, None)

        # Hook a method.
        self.type_manager.add_pre(EmptySubOfFoo, 'foogle', hook)

        # Make sure that the method was added to the class dictionary.
        method = EmptySubOfFoo.__dict__.get('foogle')
        self.assertNotEqual(method, None)

        # Call the method that we have hooked.
        esof.foogle()

        # Make sure that the hook was called.
        self.assertEqual(len(l), 1)

        # Remove the hook.
        self.type_manager.remove_pre(EmptySubOfFoo, 'foogle', hook)

        # Call the method that we have hooked.
        esof.foogle()

        # Make sure that the hook was NOT called.
        self.assertEqual(len(l), 1)

        # Make sure that we didn't put the original method back onto
        # 'EmptySubOfFoo'(since it didn't override it in the first place).
        method = EmptySubOfFoo.__dict__.get('foogle')
        self.assertEqual(method, None)

        return

    def test_type_manager_hierarchy(self):
        """ type manager hierarchy """

        # Register an adapter Foo->Bar.
        self.type_manager.register_type_adapters(FooToBarAdapterFactory(), Foo)

        # Create an empy type manager with the main type manager as its
        # parent.
        type_manager = TypeManager(parent=self.type_manager)

        # Create a Foo.
        foo = Foo(name='fred')

        # Adapt it to a Bar.
        bar = type_manager.object_as(foo, Bar)
        self.assertNotEqual(bar, None)
        self.assertEqual(bar.blargle(), 'Foo.foogle.fred')

        return

#### EOF ######################################################################



#-------------------------------------------------------------------------------
#
#  Defines the ITemplate interface used to create templatizable object
#  hierarchies.
#
#  Written by: David C. Morrill
#
#  Date: 07/26/2007
#
#  (c) Copyright 2007 by Enthought, Inc.
#
#-------------------------------------------------------------------------------

""" Defines the ITemplate interface used to create templatizable object
    hierarchies.
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

from traits.api \
    import Interface

#-------------------------------------------------------------------------------
#  'ITemplate' interface:
#-------------------------------------------------------------------------------

class ITemplate ( Interface ):
    """ Defines the ITemplate interface used to create templatizable object
        hierarchies.
    """

    def names_from_template ( self ):
        """ Returns a list of **TemplateDataName** objects, one for each
            bindable data source contained within the template. Each
            **TemplateDataName** object supports unresolved bindings in
            addition to resolved bindings. Also, a **TemplateDataName** may be
            marked as *optional*, meaning the if the name remains unresolved,
            the application can take an alternative action (e.g. omit the data
            from a plot, or substitute default data, etc.).

            Returns
            -------
            A list of **TemplateDataName** objects, one for each bindable data
            source contained in the template.
        """

    def object_from_template ( self ):
        """ Activates the object from its template data.

            Returns
            -------
            The original object.
        """

    def template_from_object ( self ):
        """ Returns a *templatized* version of the object that is safe for
            serialization.

            Returns
            -------
            A new object of the same class as the original.
        """

    def activate_template ( self ):
        """ Converts all contained 'TDerived' objects to real objects using the
            template traits of the object.

            Returns
            -------
            None
        """




""" The base class for adaptable objects. """


# Enthought library imports.
from traits.api import HasTraits, Instance

# Local imports.
from .adapter_manager import AdapterManager


class Adaptable(HasTraits):
    """ The base class for adaptable objects. """

    #### 'Adaptable' interface ################################################

    # The adapter manager in effect for the object.
    adapter_manager = Instance(AdapterManager)

    ###########################################################################
    # 'Adaptable' interface.
    ###########################################################################

    def adapt(self, target_class, *args, **kw):
        """ Returns True if the factory can produce an appropriate adapter. """

        if self.adapter_manager is not None:
            adapter = self.adapter_manager.adapt(
                self, target_class, *args, **kw
            )

        else:
            adapter = None

        return adapter

#### EOF ######################################################################



#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought naming package component>
#------------------------------------------------------------------------------
""" Tests the Python namespace context. """


# Enthought library imports.
from apptools.naming.api import PyContext

# Local imports.
from context_test_case import ContextTestCase


class PyContextTestCase(ContextTestCase):
    """ Tests the Python namespace context. """

    ###########################################################################
    # 'ContextTestCase' interface.
    ###########################################################################

    def create_context(self):
        """ Creates the context that we are testing. """

        return PyContext(namespace={})

#### EOF ######################################################################



""" Help Plugin API
    :Copyright: 2008, Enthought Inc.
    :License: BSD
    :Author: Janet Swisher
"""
# This software is provided without warranty under the terms of the BSD
# license included in AppTools/trunk/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!

from .help_code import HelpCode
from .help_doc import HelpDoc
from .help_plugin import HelpPlugin
from .i_help_code import IHelpCode
from .i_help_doc import IHelpDoc



#-------------------------------------------------------------------------------
#
#  A concrete implementation of the ITemplateDataContext interface intended to
#  be used for creating the *output_data_context* value of an
#  **ITemplateDataNameItem** implementation (although they are not required to
#  use it).
#
#  Written by: David C. Morrill
#
#  Date: 07/29/2007
#
#  (c) Copyright 2007 by Enthought, Inc.
#
#-------------------------------------------------------------------------------

""" A concrete implementation of the ITemplateDataContext interface intended to
    be used for creating the *output_data_context* value of an
    **ITemplateDataNameItem** implementation (although they are not required to
    use it).
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

from traits.api\
    import HasPrivateTraits, Dict, Str, Any, Property, provides, \
           cached_property

from apptools.template.itemplate_data_context \
    import ITemplateDataContext, ITemplateDataContextError

#-------------------------------------------------------------------------------
#  'TemplateDataContext' class:
#-------------------------------------------------------------------------------

class TemplateDataContext ( HasPrivateTraits ):
    """ A concrete implementation of the ITemplateDataContext interface
        intended to be used for creating the *output_data_context* value of an
        **ITemplateDataNameItem** implementation (although they are not
        required to use it).
    """

    implements( ITemplateDataContext )

    #-- 'ITemplateDataContext' Interface Traits --------------------------------

    # The path to this data context (does not include the 'data_context_name'):
    data_context_path = Str

    # The name of the data context:
    data_context_name = Str

    # A list of the names of the data values in this context:
    data_context_values = Property # List( Str )

    # The list of the names of the sub-contexts of this context:
    data_contexts = Property # List( Str )

    #-- Public Traits ---------------------------------------------------------

    # The data context values dictionary:
    values = Dict( Str, Any )

    # The data contexts dictionary:
    contexts = Dict( Str, ITemplateDataContext )

    #-- 'ITemplateDataContext' Property Implementations ------------------------

    @cached_property
    def _get_data_context_values ( self ):
        values = list(self.values.keys())
        values.sort()
        return values

    @cached_property
    def _get_data_contexts ( self ):
        contexts = list(self.contexts.keys())
        contexts.sort()
        return contexts

    #-- 'ITemplateDataContext' Interface Implementation ------------------------

    def get_data_context_value ( self, name ):
        """ Returns the data value with the specified *name*. Raises a
            **ITemplateDataContextError** if *name* is not defined as a data
            value in the context.

            Parameters
            ----------
            name : A string specifying the name of the context data value to
                be returned.

            Returns
            -------
            The data value associated with *name* in the context. The type of
            the data is application dependent.

            Raises **ITemplateDataContextError** if *name* is not associated
            with a data value in the context.
        """
        try:
            return self.values[ name ]
        except:
            raise ITemplateDataContextError( "Value '%s' not found." % name )

    def get_data_context ( self, name ):
        """ Returns the **ITemplateDataContext** value associated with the
            specified *name*. Raises **ITemplateDataContextError** if *name* is
            not defined as a data context in the context.

            Parameters
            ----------
            name : A string specifying the name of the data context to be
                returned.

            Returns
            -------
            The **ITemplateDataContext** associated with *name* in the context.

            Raises **ITemplateDataContextError** if *name* is not associated
            with a data context in the context.
        """
        try:
            return self.context[ name ]
        except:
            raise ITemplateDataContextError( "Context '%s' not found." % name )




from traits.api import Event, Interface, Str


class ISelectionProvider(Interface):
    """ Source of selections. """

    #: Unique ID identifying the provider.
    provider_id = Str()

    #: Event triggered when the selection changes.
    #: The content of the event is an :class:`~.ISelection` instance.
    selection = Event

    def get_selection(self):
        """ Return the current selection.

        Returns:
            selection -- ISelection
                Object representing the current selection.
        """

    def set_selection(self, items, ignore_missing=False):
        """ Set the current selection to the given items.

        If ``ignore_missing`` is ``True``, items that are not available in the
        selection provider are silently ignored. If it is ``False`` (default),
        an :class:`~.ValueError` should be raised.

        Arguments:
            items -- list
                List of items to be selected.

            ignore_missing -- bool
                If ``False`` (default), the provider raises an exception if any
                of the items in ``items`` is not available to be selected.
                Otherwise, missing elements are silently ignored, and the rest
                is selected.
        """
